function trigger_box(block)
% Level-2 M file S-Function for unit delay demo.
%   Copyright 1990-2009 The MathWorks, Inc.
%   $Revision: 1.1.6.1 $ 

  setup(block);
  
%endfunction

function setup(block)
  
  block.NumDialogPrms  = 0;
 
  
  %% Register number of input and output ports
  block.NumInputPorts  = 3;
  block.NumOutputPorts = 1;

  %% Setup functional port properties to dynamically
  %% inherited.
  block.SetPreCompInpPortInfoToDynamic;
  block.SetPreCompOutPortInfoToDynamic;
 
  block.InputPort(1).Dimensions        = 1;
  block.InputPort(2).Dimensions        = 1;
  block.InputPort(3).Dimensions        = 1;
  block.InputPort(1).DirectFeedthrough = false;
  block.InputPort(2).DirectFeedthrough = false;
  block.InputPort(3).DirectFeedthrough = false;
  block.OutputPort(1).Dimensions       = 1;
  
  %% Set block sample time to inherited
  
  block.SampleTimes = [-1 0];
  
  %% Set the block simStateCompliance to default (i.e., same as a built-in block)
  block.SimStateCompliance = 'DefaultSimState';
  

  %% Register methods
  block.RegBlockMethod('PostPropagationSetup',    @DoPostPropSetup);
  block.RegBlockMethod('InitializeConditions',    @InitConditions);  
  block.RegBlockMethod('Outputs',                 @Output);  
  block.RegBlockMethod('Update',                  @Update);  
  

  
%endfunction

function DoPostPropSetup(block)

  %% Setup Dwork
  block.NumDworks = 1;
  block.Dwork(1).Name = 'state'; 
  block.Dwork(1).Dimensions      =2;
  block.Dwork(1).DatatypeID      = 0;
  block.Dwork(1).Complexity      = 'Real';
  block.Dwork(1).UsedAsDiscState = true;

%endfunction

function InitConditions(block)

  %% Initialize Dwork
  n = block.Dwork(1).Dimensions;
  block.Dwork(1).Data(1) = 0;       %output state active
  block.Dwork(1).Data(2) = 1;       %latch state active
  
  
%endfunction

function Output(block)

  output_state = block.Dwork(1).Data(1);
  block.OutputPort(1).Data = output_state;
 
 
%endfunction

function Update(block)
    output_state  = block.Dwork(1).Data(1);
    latch_state   = block.Dwork(1).Data(2);
    reset_input   = block.InputPort(1).Data;
    current_input = block.InputPort(2).Data;
    freq_input    = block.InputPort(3).Data;
    
    if (latch_state==1)&(reset_input==1)
        latch_state  = 0;
        output_state = 1;
    end;
    
    if(latch_state==0)&(current_input==1)&(output_state==1)
        latch_state  = 1;
        output_state = 0;
    end;
    
    if(latch_state==0)&(freq_input==1)&(output_state==1)
        latch_state  = 1;
        output_state = 0;
    end;
    
    block.Dwork(1).Data(1)=output_state;
    block.Dwork(1).Data(2)=latch_state;
    
  
%endfunction

